@@ -1,6 +1,6 @@
-;;; mu4e-headers.el --- Message headers -*- lexical-binding: t; coding:utf-8 -*-
+;;; mu4e-headers.el --- Message headers -*- lexical-binding: t  -*-
 
-;; Copyright (C) 2011-2024 Dirk-Jan C. Binnema
+;; Copyright (C) 2011-2025 Dirk-Jan C. Binnema
 
 ;; Author: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 ;; Maintainer: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
@@ -53,7 +53,6 @@
 (declare-function mu4e-view       "mu4e-view")
 (declare-function mu4e--main-view  "mu4e-main")
 
-
 ;;; Configuration
 
 (defgroup mu4e-headers nil
@@ -217,7 +216,6 @@ When it is showing, respectively, From: or To:. It is a cons cell
 (defvar mu4e-headers-personal-mark  '("p" . "Ⓟ") "Personal.")
 (defvar mu4e-headers-calendar-mark  '("c" . "Ⓒ") "Calendar invitation.")
 
-
 ;;;; Graph drawing
 
 (defvar mu4e-headers-thread-mark-as-orphan 'first
@@ -386,8 +384,6 @@ Returns the docid, or nil if there is none."
       (goto-char point))
     (get-text-property (line-beginning-position) 'docid)))
 
-
-
 (defun mu4e~headers-goto-docid (docid &optional to-mark)
   "Go to beginning of the line with DOCID.
 Nil if it cannot be found. If the optional TO-MARK is
@@ -550,9 +546,9 @@ e.g. \"mutt\"."
 (defsubst mu4e~headers-custom-field-value (msg field)
   "Show custom header FIELD for MSG, or raise error if not found."
   (let* ((item (or (assoc field mu4e-header-info-custom)
-                   (mu4e-error "field %S not found" field)))
+                   (mu4e-error "Field %S not found" field)))
          (func (or (plist-get (cdr-safe item) :function)
-                   (mu4e-error "no :function defined for field %S %S"
+                   (mu4e-error "No :function defined for field %S %S"
                                field (cdr item)))))
     (funcall func msg)))
 
@@ -606,7 +602,7 @@ the correct column for display."
                             sum w
                             until (equal f field))))
       (setq val (string-trim-right val))
-      (if (> width (length val))
+      (if (> width (string-width val))
           (setq val (concat val " "))
         (setq val
               (concat
@@ -669,7 +665,7 @@ This is suitable for displaying in the header view."
 
 (defsubst mu4e~headers-insert-header (msg pos)
   "Insert a header for MSG at point POS."
-  (when-let ((line (mu4e~message-header-line msg))
+  (when-let* ((line (mu4e~message-header-line msg))
              (docid (plist-get msg :docid)))
     (goto-char pos)
     (insert
@@ -804,15 +800,13 @@ present, don't do anything."
 If so, do not attempt to switch buffers. This variable is to be let-bound
 to t before \"automatic\" searches.")
 
-(defun mu4e--search-execute (expr ignore-history)
+(defun mu4e--search-execute (expr &optional _ignore-history)
   "Search for query EXPR.
 
-Switch to the output buffer for the results. If IGNORE-HISTORY is
-true, do *not* update the query history stack."
+Switch to the output buffer for the results."
   (let* ((buf (mu4e-get-headers-buffer nil t))
          (view-window mu4e~headers-view-win)
          (inhibit-read-only t)
-         (rewritten-expr (funcall mu4e-query-rewrite-function expr))
          (maxnum (unless mu4e-search-full mu4e-search-results-limit)))
     (with-current-buffer buf
       ;; NOTE: this resets all buffer-local variables, including
@@ -820,13 +814,8 @@ true, do *not* update the query history stack."
       ;; headers buffer already exists when `mu4e-get-headers-buffer'
       ;; is called.
       (mu4e-headers-mode)
-      (setq mu4e~headers-view-win view-window)
-      (unless ignore-history
-        ;; save the old present query to the history list
-        (when mu4e--search-last-query
-          (mu4e--search-push-query mu4e--search-last-query 'past)))
-      (setq mu4e--search-last-query rewritten-expr)
-      (setq list-buffers-directory rewritten-expr)
+      (setq mu4e~headers-view-win view-window
+            list-buffers-directory expr)
       (mu4e--modeline-update))
 
     ;; when the buffer is already visible, select it; otherwise,
@@ -837,7 +826,7 @@ true, do *not* update the query history stack."
     (mu4e~headers-clear mu4e~search-message)
     (setq mu4e~headers-search-start (float-time))
     (mu4e--server-find
-     rewritten-expr
+     expr
      mu4e-search-threads
      mu4e-search-sort-field
      mu4e-search-sort-direction
@@ -846,7 +835,8 @@ true, do *not* update the query history stack."
      mu4e-search-include-related)))
 
 (defun mu4e~headers-benchmark-message (count)
-  "Get some report message for messaging search and rendering speed."
+  "Get some report message for messaging search and rendering speed.
+COUNT is the number of messages found."
   (if (and mu4e-headers-report-render-time
            mu4e~headers-search-start
            mu4e~headers-render-start
@@ -861,7 +851,7 @@ true, do *not* update the query history stack."
     ""))
 
 (defun mu4e~headers-found-handler (count)
-  "Create one=line description of the COUNT of headers found."
+  "Create one-line description of the COUNT of headers found."
   (when (buffer-live-p (mu4e-get-headers-buffer))
     (with-current-buffer (mu4e-get-headers-buffer)
       (save-excursion
@@ -1033,51 +1023,64 @@ true, do *not* update the query history stack."
      )))
 
 ;;; Headers-mode and mode-map
+(defun mu4e~header-line-click (sortable threads)
+  "Return function for header-line clicks.
+If SORTABLE, handle the sorting; otherwise show a message that
+you cannot sort by this field.
+
+If THREADS is non-nil, give a more informative error message."
+  (if (not sortable)
+      (if threads
+          (lambda (&optional _e)
+            (interactive "e")
+            (mu4e-message "With threading, you can only sort by date"))
+        (lambda (&optional  _e)
+          (interactive "e")
+          (mu4e-message "Field is not sortable")))
+    (lambda (&optional e)
+      (interactive "e")
+      ;; getting the field, inspired by `tabulated-list-col-sort'
+      (let* ((obj (posn-object (event-start e)))
+             (field (and obj
+                         (get-text-property 0 'field (car obj)))))
+        ;; "t": if we're already sorted by field, the
+        ;; sort-order is changed
+        (mu4e-search-change-sorting field t)))))
 
 (defun mu4e~header-line-format ()
   "Get the format for the header line."
-  (let ((uparrow   (if mu4e-use-fancy-chars " ▲" " ^"))
-        (downarrow (if mu4e-use-fancy-chars " ▼" " V")))
+  (let* ((plist (mu4e-server-last-query)) ;; info about the last query
+         (reverse (plist-get plist :descending))
+         (threads (plist-get plist :threads))
+         ;; with threads enabled,  we can only sort by ;date
+         (sort-field (if threads :date (plist-get plist :sort-field)))
+         (fields (append mu4e-header-info mu4e-header-info-custom))
+         (arrow (if reverse
+                    (if mu4e-use-fancy-chars " ▼" " V")
+                  (if mu4e-use-fancy-chars " ▲" " ^"))))
     (cons
      (make-string
       (+ mu4e--mark-fringe-len (floor (fringe-columns 'left t))) ?\s)
      (mapcar
       (lambda (item)
-        (let* (;; with threading enabled, we're necessarily sorting by date.
-               (sort-field (if mu4e-search-threads
-                               :date mu4e-search-sort-field))
-               (field (car item)) (width (cdr item))
-               (info (cdr (assoc field
-                                 (append mu4e-header-info
-                                         mu4e-header-info-custom))))
-               (sortable (plist-get info :sortable))
-               ;; if sortable, it is either t (when field is sortable itself)
-               ;; or a symbol (if another field is used for sorting)
-               (this-field (when sortable (if (booleanp sortable)
-                                              field
-                                            sortable)))
+        (let* ((field (car item)) (width (cdr item))
+               (info (cdr (assoc field fields)))
+               (sortable-info (plist-get info :sortable))
+               ;; the effective sort-field for this field is as per its info;
+               ;; if t, it's the field itself; otherwise it's either some
+               ;; _other_ field (for fields which are sorted by some other field), or nil
+               ;; for fields that cannot be sorted.
+               (field-sort-field
+                (if (eq sortable-info t) field sortable-info))
+               ;; only if we're actually looking at if for this column
+               (current-sort-field (and (eq field-sort-field sort-field) field-sort-field))
                (help (plist-get info :help))
                ;; triangle to mark the sorted-by column
-               (arrow
-                (when (and sortable (eq this-field sort-field))
-                  (if (eq mu4e-search-sort-direction 'descending)
-                      downarrow
-                    uparrow)))
-               (name (concat (plist-get info :shortname) arrow))
+               (name (concat (plist-get info :shortname)
+                             (if current-sort-field arrow "")))
                (map (make-sparse-keymap)))
-          (when sortable
-            (define-key map [header-line mouse-1]
-                        (lambda (&optional e)
-                          ;; getting the field, inspired by
-                          ;; `tabulated-list-col-sort'
-                          (interactive "e")
-                          (let* ((obj (posn-object (event-start e)))
-                                 (field
-                                  (and obj
-                                       (get-text-property 0 'field (car obj)))))
-                            ;; "t": if we're already sorted by field, the
-                            ;; sort-order is changed
-                            (mu4e-search-change-sorting field t)))))
+          (define-key map [header-line mouse-1]
+                      (mu4e~header-line-click field-sort-field threads))
           (concat
            (propertize
             (if width
@@ -1086,26 +1089,30 @@ true, do *not* update the query history stack."
               name)
             'face (when arrow 'bold)
             'help-echo help
-            'mouse-face (when sortable 'highlight)
-            'keymap (when sortable map)
+            'mouse-face (when field-sort-field 'highlight)
+            'keymap map
             'field field) " ")))
       mu4e-headers-fields))))
 
 (defun mu4e~headers-maybe-auto-update ()
-  "Update the current headers buffer after indexing changes.
+  "Update the current headers buffer after indexing.
+
 Furthermore, `mu4e-headers-auto-update' is non-nil and there is
-no user-interaction ongoing."
-  (when (and mu4e-headers-auto-update ;; must be set
-             mu4e-index-update-status
-             (not (mu4e-get-view-buffer)) ;; not when viewing a message
-             (not (zerop (plist-get mu4e-index-update-status :updated)))
-             ;; NOTE: `mu4e-mark-marks-num' can return nil. Is that intended?
-             (zerop (or (mu4e-mark-marks-num) 0)) ;; non active marks
-             (not (active-minibuffer-window)))    ;; no user input only
-    ;; rerun search if there's a live window with search results;
-    ;; otherwise we'd trigger a headers view from out of nowhere.
-    (when (and (buffer-live-p (mu4e-get-headers-buffer))
-               (window-live-p (get-buffer-window (mu4e-get-headers-buffer) t)))
+no user-interaction ongoing.
+
+We only update headers when quite a few conditions are true --
+see the code."
+  (when-let* ((hdrsbuf (mu4e-get-headers-buffer)))
+    (when (and mu4e-headers-auto-update ;; must be set
+               mu4e-index-update-status
+               (not (mu4e-get-view-buffer)) ;; not when viewing a message
+               (not (zerop (plist-get mu4e-index-update-status :updated)))
+               (buffer-live-p hdrsbuf)
+               (window-live-p (get-buffer-window hdrsbuf t))
+               ;; don't disturb marks.
+               (zerop (or (with-current-buffer hdrsbuf (mu4e-mark-marks-num)) 0))
+               (not (active-minibuffer-window))) ;; no user input only
+      ;; when all that is true, rerun the current query.
       (let ((mu4e--search-background t))
         (mu4e-search-rerun)))))
 
@@ -1261,7 +1268,7 @@ corresponding header."
 
 
 (defun mu4e-headers-find-if (func &optional backward)
-  "Move to the header for which FUNC returns non-nil.
+  "Move to the header where FUNC yields non-nil.
 if BACKWARD is non-nil, search backwards.
 
 FUNC receives one argument, the message s-expression for the
@@ -1427,16 +1434,13 @@ If SUBTHREAD is non-nil, only apply to subthread."
   (unless (eq major-mode 'mu4e-headers-mode)
     (mu4e-error "Must be in mu4e-headers-mode (%S)" major-mode))
   (let* ((msg (mu4e-message-at-point))
-         (path (mu4e-message-field msg :path))
-         (_exists (or (file-readable-p  path)
-                      (mu4e-warn "No message at %s" path)))
          (docid (or (mu4e-message-field msg :docid)
                     (mu4e-warn "No message at point")))
          (mark-as-read
           (if (functionp mu4e-view-auto-mark-as-read)
               (funcall mu4e-view-auto-mark-as-read msg)
             mu4e-view-auto-mark-as-read)))
-    (when-let ((buf (mu4e-get-view-buffer (current-buffer) nil)))
+    (when-let* ((buf (mu4e-get-view-buffer (current-buffer) nil)))
       (with-current-buffer buf
         (mu4e-loading-mode 1)))
     (mu4e--server-view docid mark-as-read)))
