@@ -1,6 +1,6 @@
 ;;; mu4e-helpers.el --- Helper functions -*- lexical-binding: t -*-
 
-;; Copyright (C) 2022-2024  Dirk-Jan C. Binnema
+;; Copyright (C) 2022-2025 Dirk-Jan C. Binnema
 
 ;; Author: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 ;; Maintainer: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
@@ -68,8 +68,7 @@ might want to add something like the following in your configuration.
   :group 'mu4e)
 
 (defcustom mu4e-read-option-use-builtin t
-  "Whether to use mu4e's traditional completion for
-`mu4e-read-option'.
+  "Whether to use mu4e's traditional completion for `mu4e-read-option'.
 
 If nil, use the value of `mu4e-completing-read-function', integrated
 into mu4e.
@@ -259,7 +258,7 @@ Return the matching choice value (cdr of the cell)."
                         (mapconcat #'car choices ", "))))
     (while (not chosen)
       (message nil) ;; this seems needed...
-      (when-let ((kar (read-char-exclusive prompt)))
+      (when-let* ((kar (read-char-exclusive prompt)))
         (when (eq kar ?\e) (keyboard-quit)) ;; `read-char-exclusive' is a C
                                             ;; function and doesn't check for
                                             ;; `keyboard-quit', there we need to
@@ -289,7 +288,7 @@ user can then choose by typing CHAR.  Example:
 User now will be presented with a list: \"Choose an animal:
    [M]onkey, [G]nu, [x]Moose\".
 
-If optional character KEY is provied, use that instead of asking
+If optional character KEY is provided, use that instead of asking
 the user.
 
 Function returns the value (cdr) of the matching cell."
@@ -521,30 +520,38 @@ Or go to the top level if there is none."
           ('mu4e-view-mode    "(mu4e)Message view")
           (_                  "mu4e"))))
 
+;;; Emacs bookmarks
+(defcustom mu4e-emacs-bookmark-policy 'message
+  "The policy to decide what kind of Emacs bookmark to create.
 
-;;; bookmarks
-(defun mu4e--make-bookmark-record ()
-  "Create a bookmark for the message at point."
-  (let* ((msg     (mu4e-message-at-point))
-         (subject (or (plist-get msg :subject) "No subject"))
-         (date	  (plist-get msg :date))
-         (date	  (if date (format-time-string "%F: " date) ""))
-         (title	  (format "%s%s" date subject))
-         (msgid	  (or (plist-get msg :message-id)
-                      (mu4e-error
-                       "Cannot bookmark message without message-id"))))
-    `(,title
-      ,@(bookmark-make-record-default 'no-file 'no-context)
-      (message-id . ,msgid)
-      (handler    . mu4e--jump-to-bookmark))))
+This applies to the Emacs bookmark system command `bookmark-set'.
 
-(declare-function mu4e-view-message-with-message-id "mu4e-view")
-(declare-function mu4e-message-at-point             "mu4e-message")
+The policy is some symbol; you have the following choices:
+- `message': bookmark the message at point
+- `query': bookmark the most recent query
+- `ask': ask the user interactively"
+  :type '(choice
+          (const :tag "Bookmark query at point" message)
+          (const :tag "Bookmark the last query" query)
+          (const :tag "Ask user" ask))
+  :group 'mu4e)
 
-(defun mu4e--jump-to-bookmark (bookmark)
-  "View the message referred to by BOOKMARK."
-  (when-let ((msgid (bookmark-prop-get bookmark 'message-id)))
-    (mu4e-view-message-with-message-id msgid)))
+(defun mu4e--popup-lisp-buffer (bufname data)
+  "Show or hide an s-expression string in a popup-buffer.
+BUFNAME is the name of the buffer, and DATA is lisp-data, if any."
+  (if-let* ((win (get-buffer-window bufname)))
+      (delete-window win)
+    (when data
+      (when (buffer-live-p bufname)
+        (kill-buffer bufname))
+      (with-current-buffer-window
+          (get-buffer-create bufname) nil nil
+        ;; sadly, the default indentation for plists
+        ;; is not very nice, and I failed to overwrite it
+        (lisp-mode)
+        (insert (pp-to-string data))
+        ;; add basic `quit-window' bindings
+        (view-mode 1)))))
 
 ;;; Macros
 
@@ -552,7 +559,12 @@ Or go to the top level if there is none."
   "Set VAR to VAL if VAR is nil."
   `(unless ,var (setq ,var ,val)))
 
+
 ;;; Misc
+(defun mu4e--fun-val (val)
+  "If var is a function, return (funcall VAL), otherwise return VAL."
+  (if (functionp val) (funcall val) val))
+
 (defun mu4e-join-paths (directory &rest components)
   "Append COMPONENTS to DIRECTORY and return the resulting string.
 
@@ -566,17 +578,46 @@ COMPONENTS."
 
 (defun mu4e-string-replace (from-string to-string in-string)
   "Replace FROM-STRING with TO-STRING in IN-STRING each time it occurs.
-Mu4e version of emacs 28's string-replace."
+Mu4e's version of Emacs 28's `string-replace'."
   (replace-regexp-in-string (regexp-quote from-string)
                             to-string in-string nil 'literal))
 
 (defun mu4e-plistp (object)
   "Non-nil if and only if OBJECT is a valid plist.
-
 This is mu4e's version of Emacs 29's `plistp'."
   (let ((len (proper-list-p object)))
     (and len (zerop (% len 2)))))
 
+(defun mu4e-plist-filter (plist pred)
+  "Return a plist for which a call to pred is non-nil.
+PLIST is some plist, and PRED is a function taking a key and
+value."
+  (let (p)
+    (while plist
+      (let ((key (car plist)) (val (cadr plist)))
+           (when (funcall pred key val)
+             (setq p (plist-put p key val))))
+      (setq plist (cddr plist)))
+    p))
+
+(defun mu4e-plist-remove (plist prop)
+  "Remove PROP from PLIST.
+Returns the updated PLIST."
+  (mu4e-plist-filter plist (lambda (k _v) (not (eq k prop)))))
+
+(defun mu4e-plist-remove-nils (plist)
+  "Remove all properties with value nil from PLIST."
+  (mu4e-plist-filter plist (lambda (_k v) v)))
+
+(defun mu4e-plist-put-many (plist &rest keyvals)
+  "Like `plist-put', but allow for multiple key-value pairs.
+PLIST is a property list, and KEYVALS are key value ... ."
+  (let ((p plist))
+    (while keyvals
+      (setq p (plist-put p (car keyvals) (cadr keyvals)))
+      (setq keyvals (cddr keyvals)))
+    p))
+
 (defun mu4e--message-hide-headers ()
   "Hide headers based on the `message-hidden-headers' variable.
 This is mu4e's version of the post-emacs-28 `message-hide-headers',
